#
# To use this makefile from the command line, it is recommended to
# create the following alias:
#
#     alias make='make -f Makefile-qnx.mk'
#
# To make a fresh start, just run the following command:
#
#     make distclean depend all
#
# If that doesn't work, the following should always work:
#
#     rm obj/depend
#     make distclean depend all
#
# The Watcom compilers for QNX 4.25 do not generate dependency files
# on the fly.  Instead, this Makefile uses the old, semi-automatic
# approach where the user is required to run either the fresh start
# command (above) or the following command when dependencies change:
#
#     make depend
#
# This Makefile requires the mkdep script which can be downloaded from
# the following URL:
#
#     ftp://ftp.qnx.com/usr/free/qnx4/os/development/mkdep.gz
#

LIBRARY = lib/cgul.lib

CPPFLAGS = -Iinclude
CFLAGS = -Q $(DCFLAGS)
LDFLAGS =

AR = ar
ARFLAGS = cruq

# Automatically generate the list of source and object files.
SRCS := $(filter-out src/main.c,$(wildcard src/*.c))
OBJS := $(patsubst src/%.c,obj/%.o,$(SRCS))

# Use "make debug" to build a debugging version.
ifdef DEBUG
    DCFLAGS = -g -Od
else
    DCFLAGS = -O -DNDEBUG
endif

all: $(LIBRARY)
	@echo "Done."

$(LIBRARY): $(OBJS)
	$(AR) $(ARFLAGS) $@ $?

# Make $(DEPS) file.  Some rewriting has to occur because mkdep
# assumes the object file will be created in the same directory as the
# source file, but this Makefile creates object files under the "obj"
# directory.
depend:
	@printf "Making dependencies ... "
	@mkdep -f $(DEPS) $(CPPFLAGS) src/main.c $(SRCS)
	@sed -e 's|^.*/\(.*\):|obj/\1:|' $(DEPS) > $(DEPS).tmp
	@mv $(DEPS).tmp $(DEPS)
	@echo "Done."

debug:
	@$(MAKE) -f Makefile-qnx.mk DEBUG=1

# Remove everything except the $(DEPS) file.
clean:
	-$(RM) $(LIBRARY) obj/*.o *.err

# Remove everything including the $(DEPS) file.
distclean: clean
	-$(RM) $(DEPS)

obj/%.o: src/%.c
	$(CC) $(CFLAGS) $(CPPFLAGS) -o $@ -c $<

DEPS = obj/depend
-include $(DEPS)

.PHONY: all clean distclean debug depend
