#ifndef CGUL_BITWISE_TRIE_P_H
#define CGUL_BITWISE_TRIE_P_H

#include "cgul_common.h"
#include "cgul_bitwise_trie.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_bitwise_trie</code> class.
 */
struct cgul_bitwise_trie {

    /**
     * Size of the trie.  This is the count of the number of nodes
     * (or index/value pairs) held by the trie.
     */
    unsigned long int size;

    /**
     * Root node of the tree.  It makes the implementation easier if
     * the root node is empty.  It's left child will be the subtree
     * comprised of nodes that have even indexes, and it's right child
     * will be the subtree comprised of nodes that have odd indexes.
     */
    cgul_bitwise_trie_node_t root;

    /**
     *  The oldest node in the trie.  This is the first node that was
     *  inserted.
     */
    cgul_bitwise_trie_node_t oldest;

    /**
     *  The youngest node in the trie.  This is the last node that was
     *  inserted.
     */
    cgul_bitwise_trie_node_t youngest;

    /**
     * For efficiency, the <code>cgul_bitwise_trie</code> can keep a
     * cache of nodes that can be reused.  In many situations, this
     * can greatly reduce the number of calls to <code>malloc()</code>
     * which can greatly improve performance and reduce memory
     * fragmentation.
     */
    cgul_cache_t cache;

};

CGUL_END_C

#endif  /*  CGUL_BITWISE_TRIE_P_H  */
