#ifndef CGUL_BLOCK_READER_MEMORY_P_H
#define CGUL_BLOCK_READER_MEMORY_P_H

#include "cgul_block_reader_memory.h"
#include "cgul_block_reader_p.h"

CGUL_BEGIN_C

/** Data encapsulated by the <code>cgul_block_reader_memory</code> class. */
struct cgul_block_reader_memory {

    /**
     * Inherit from the <code>cgul_block_reader</code> base class.
     */
    struct cgul_block_reader super;

    /**
     * Input buffer from which blocks are read.  We do *not* own this
     * buffer.  The client is responsible for freeing it if necessary.
     * Note that it is legal for the client to set the input buffer to
     * <code>NULL</code> provided the client also sets
     * <code>buffer_offset</code> to zero.
     */
    unsigned char* buffer;

    /**
     * The number of bytes held by the input buffer.
     */
    size_t buffer_size;

    /**
     * Index into the input buffer of the next byte to read.
     */
    size_t buffer_offset;

    /**
     * Name to present to the user to help identify which buffer is
     * being read.  This is primarily used for error reporting.x
     */
    char* presentation_name;

};

CGUL_END_C

#endif  /*  CGUL_BLOCK_READER_MEMORY_P_H  */
