#include "cgul_endian.h"
#include "cgul_int.h"

int
cgul_endian__is_big_endian(cgul_exception_t* cex)
{
    int rv = 0;
    cgul_uint8_t buf[4] = {0x01, 0x23, 0x45, 0x67};
    cgul_uint32_t* x = (cgul_uint32_t*)buf;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test the exact order of the bytes in x. */
    if (*x == 0x01234567) {
        rv = 1;
    }

 out:
    return rv;
}

int
cgul_endian__is_little_endian(cgul_exception_t* cex)
{
    int rv = 0;
    cgul_uint8_t buf[4] = {0x01, 0x23, 0x45, 0x67};
    cgul_uint32_t* x = (cgul_uint32_t*)buf;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test the exact order of the bytes in x. */
    if (*x == 0x67452301) {
        rv = 1;
    }

 out:
    return rv;
}

void
cgul_endian__swap(cgul_exception_t* cex,
                  void* value,
                  size_t value_size)
{
    size_t i = 0;
    size_t j = 0;
    unsigned char tmp = 0;
    unsigned char* ptr = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Sanity check. */
    if (!value || !value_size) {
        goto out;
    }

    /* It is more convenient for the user if "value" is a void
     * pointer, but the loop below works one byte at a time. */
    ptr = (unsigned char*)value;

    /*
     * To convert between big endian and little endian, just reverse
     * the order of the bytes.
     */

    /* Swap the bytes on both ends.  Move both indexes in toward the
     * center.  Swap those bytes.  Repeat. */
    for (i = 0, j = value_size - 1 ; i < j ; ++i, --j) {
        tmp = ptr[i];
        ptr[i] = ptr[j];
        ptr[j] = tmp;
    }

 out:
    return;
}
