#include "cgul_get_line.h"
#include "cgul_libc.h"
#include "cgul_string_p.h"

int
cgul_get_line(cgul_exception_t* cex,
              FILE* f,
              cgul_string_t s)
{
    int rv = 0;
    int rv_tmp = 0;
    char* fgets_rv = NULL;
    size_t last_char_index = 0;
    char buffer[4096];

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(f);
    assert(s);

    /* Repeatedly read from f storing each string returned in s. */
    for ( ;; ) {

        /* Get the next block of text or line. */
        fgets_rv = cgul_libc__fgets(cex, buffer, sizeof(buffer), f);
        if (*cex) {
            goto out;
        }
        if (fgets_rv == NULL) {
            /* EOF */
            break;
        }

        /* fgets() does not inform you of the length of the string that it
         * read.  So, to avoid calling strlen twice, I allow the line read
         * by fgets() to be appended to s with its possible trailing '\n'
         * character.  Then, I'll use the length that the append function
         * calculated to chop off any offending '\n' character. */
        cgul_string__append(cex, s, buffer);
        if (*cex) {
            goto out;
        }

        /* Don't save "success" in "rv" directly in case an exception
         * is thrown in the interim. */
        rv_tmp = 1;

        last_char_index = s->length - 1;
        if( s->value[last_char_index] == '\n' ) {
            s->value[last_char_index] = '\0';
            --(s->length);
            /* EOL */
            break;
        }
    }

    rv = rv_tmp;

 out:

    return rv;
}

int
cgul_get_line_relaxed(cgul_exception_t* cex,
                      FILE* f,
                      cgul_string_t s)
{
    int rv = 0;
    int rv_tmp = 0;
    int c = EOF;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(f);
    assert(s);

    for (;;) {

        /* Read the next character. */
        c = cgul_libc__fgetc(cex, f);
        if (*cex) {
            goto out;
        }
        if (c == EOF) {
            break;
        }

        /* Don't save "success" in "rv" directly incase an exception
         * is thrown in the interim. */
        rv_tmp = 1;

        /*
         * Check for the EOL sequences: '\n', '\r', or "\r\n"
         */
        if( c == '\n' ) {
            break;
        } else if( c == '\r' ) {
            /* It could just be an isolated '\r', or it could be the
             * "\r\n" sequence. */
            c = getc(f);
            if (c == EOF) {
                if (ferror(f)) {
                    *cex = cgul_exception__new("cgul_get_line_relaxed: "
                                               "getc: %s",
                                               strerror(errno));
                    goto out;
                }
                break;
            }
            if( c != '\n' ) {
                ungetc(c, f);
            }
            /* EOL */
            break;
        }

        /* Append the character to the cgul_string. */
        cgul_string__append_char(cex, s, c);
    }

    /* Success. */
    rv = rv_tmp;

 out:

    return rv;
}
