#include "cgul_hex_dump_p.h"
#include "cgul_defensive.h"
#include "cgul_libc.h"
#include "cgul_string_p.h"

static const char* cgul_hex_map[256] = {
    " 00",   " 01",   " 02",   " 03",   " 04",   " 05",   " 06",   " 07",
    " 08",   " 09",   " 0a",   " 0b",   " 0c",   " 0d",   " 0e",   " 0f",
    " 10",   " 11",   " 12",   " 13",   " 14",   " 15",   " 16",   " 17",
    " 18",   " 19",   " 1a",   " 1b",   " 1c",   " 1d",   " 1e",   " 1f",
    " 20",   " 21",   " 22",   " 23",   " 24",   " 25",   " 26",   " 27",
    " 28",   " 29",   " 2a",   " 2b",   " 2c",   " 2d",   " 2e",   " 2f",
    " 30",   " 31",   " 32",   " 33",   " 34",   " 35",   " 36",   " 37",
    " 38",   " 39",   " 3a",   " 3b",   " 3c",   " 3d",   " 3e",   " 3f",
    " 40",   " 41",   " 42",   " 43",   " 44",   " 45",   " 46",   " 47",
    " 48",   " 49",   " 4a",   " 4b",   " 4c",   " 4d",   " 4e",   " 4f",
    " 50",   " 51",   " 52",   " 53",   " 54",   " 55",   " 56",   " 57",
    " 58",   " 59",   " 5a",   " 5b",   " 5c",   " 5d",   " 5e",   " 5f",
    " 60",   " 61",   " 62",   " 63",   " 64",   " 65",   " 66",   " 67",
    " 68",   " 69",   " 6a",   " 6b",   " 6c",   " 6d",   " 6e",   " 6f",
    " 70",   " 71",   " 72",   " 73",   " 74",   " 75",   " 76",   " 77",
    " 78",   " 79",   " 7a",   " 7b",   " 7c",   " 7d",   " 7e",   " 7f",
    " 80",   " 81",   " 82",   " 83",   " 84",   " 85",   " 86",   " 87",
    " 88",   " 89",   " 8a",   " 8b",   " 8c",   " 8d",   " 8e",   " 8f",
    " 90",   " 91",   " 92",   " 93",   " 94",   " 95",   " 96",   " 97",
    " 98",   " 99",   " 9a",   " 9b",   " 9c",   " 9d",   " 9e",   " 9f",
    " a0",   " a1",   " a2",   " a3",   " a4",   " a5",   " a6",   " a7",
    " a8",   " a9",   " aa",   " ab",   " ac",   " ad",   " ae",   " af",
    " b0",   " b1",   " b2",   " b3",   " b4",   " b5",   " b6",   " b7",
    " b8",   " b9",   " ba",   " bb",   " bc",   " bd",   " be",   " bf",
    " c0",   " c1",   " c2",   " c3",   " c4",   " c5",   " c6",   " c7",
    " c8",   " c9",   " ca",   " cb",   " cc",   " cd",   " ce",   " cf",
    " d0",   " d1",   " d2",   " d3",   " d4",   " d5",   " d6",   " d7",
    " d8",   " d9",   " da",   " db",   " dc",   " dd",   " de",   " df",
    " e0",   " e1",   " e2",   " e3",   " e4",   " e5",   " e6",   " e7",
    " e8",   " e9",   " ea",   " eb",   " ec",   " ed",   " ee",   " ef",
    " f0",   " f1",   " f2",   " f3",   " f4",   " f5",   " f6",   " f7",
    " f8",   " f9",   " fa",   " fb",   " fc",   " fd",   " fe",   " ff"
};

cgul_hex_dump_t
cgul_hex_dump__new(cgul_exception_t* cex)
{
    cgul_hex_dump_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    rv = (cgul_hex_dump_t)cgul_libc__calloc(cex, 1, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    rv->suppress_new_line = 1;
    rv->printable_chars = cgul_string__new(cex);
    rv->strings = cgul_list__new(cex);
    if (*cex) {
        goto out;
    }

 out:

    /* Clean up any partially instantiated objects. */
    if (*cex && rv) {
        cgul_hex_dump__delete(rv);
        rv = NULL;
    }

    return rv;
}

void
cgul_hex_dump__delete(cgul_hex_dump_t hd)
{
    cgul_list_node_t node = NULL;
    cgul_exception_t local = NULL;

    if (hd) {
        if (hd->strings) {
            node = cgul_list__get_front(&local, hd->strings);
            for( ; node ; node = cgul_list_node__get_next(&local, node) ) {
                cgul_string_t s =
                    (cgul_string_t)cgul_list_node__get_value(&local, node);
                cgul_string__delete(s);
            }
            cgul_list__delete(hd->strings);
        }
        if (hd->printable_chars) {
            cgul_string__delete(hd->printable_chars);
        }
        free(hd);
    }
}

void
cgul_hex_dump__append(cgul_exception_t* cex,
                      cgul_hex_dump_t hd,
                      const void* buffer,
                      size_t buffer_length)
{
    /* Keep track of whether this method has the responsibility for
     * deleting the "value" string. */
    unsigned char c = 0;
    int we_own_value = 0;
    size_t i = 0;
    unsigned char* uc_buffer = NULL;
    /* "value" is the string that holds the result for this pass. */
    cgul_string_t value = NULL;
    cgul_exception_t local = NULL;
    char tmp[64];

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(hd);
    assert(buffer);

    if (buffer_length == 0) {
        goto out;
    }

    uc_buffer = (unsigned char*)buffer;

    value = cgul_string__new(cex);
    if (*cex) {
        goto out;
    }
    we_own_value = 1;

    cgul_list__push_back(cex, hd->strings, value);
    if (*cex) {
        goto out;
    }
    we_own_value = 0;


    /* For efficiency allocate enough extra space for this pass. */
    cgul_string__set_capacity(cex, value, ((buffer_length / 16) + 1) * 70);
    if (*cex) {
        goto out;
    }

    /* Iterate over the entire buffer. */
    while( i < buffer_length ) {

        /* Each line starts with the byte count.  You have to test if
         * "hd->j == 0" because the user can call append() more than
         * once, and on the second call, you might be in the middle of
         * the line instead of at the beginning. */
        if( hd->j == 0 ) {
            if( !hd->suppress_new_line ) {
                cgul_string__append_char(cex, value, '\n');
            }
            sprintf(tmp, "%08x", hd->byte_count);
            cgul_string__append(cex, value, tmp);
            cgul_string__append(cex, value, ":");
            hd->byte_count =
                cgul_defensive__add_unsigned(cex, hd->byte_count, 0x10);
            if (*cex) {
                goto out;
            }
        }

        /* Build up one line at a time. */
        for(  ; (hd->j < 16) && (i < buffer_length) ; ++hd->j, ++i ) {
            /* Add the middle " - " if necessary. */
            if( hd->j == 8 ) {
                cgul_string__append(cex, value, " -");
                cgul_string__append(cex, hd->printable_chars, " - ");
            }
            c = uc_buffer[i];
            /* Add the hex values directly to the value. */
            cgul_string__append(cex, value, cgul_hex_map[c]);
            /* Queue '.' if not printable; otherwise, queue the
             * character.  I added "c > 127" to help Windows to not
             * display characters xterm could not handle. */
            if( !isprint((int)c) || (c > 127) ) {
                c = '.';
            }
            cgul_string__append_char(cex, hd->printable_chars, c);
        }

        /* If we have a new line, we need to reset the hd->j counter,
           append the hd->printable_chars string, and finally reset
           the hd->printable_chars string. */
        if( hd->j >= 16 ) {
            cgul_string__append(cex, value, " ");
            cgul_string__append(cex,
                                value,
                                cgul_string__get_value(cex,
                                                       hd->printable_chars));
            cgul_string__set_value(cex, hd->printable_chars, "");
            hd->j = 0;
            hd->suppress_new_line = 0;
        }

        if (*cex) {
            goto out;
        }

    }

 out:

    /* Keep hd->total_length up to date. */
    if (value && !we_own_value) {
        hd->total_length += cgul_string__get_length(&local, value);
    }

    if (value && we_own_value) {
        cgul_string__delete(value);
    }

    return;
}

/**
 * Adjust the last line of the hex dump so that your printable
 * characters for that line are aligned with the rest of the printable
 * characters.  This method should only be called by
 * <code>cgul_hex_dump__get_value()</code>.  <code>s</code> is the
 * string you plan on returning to the user with the full hex dump.
 * It should already be complete except for the printable characters
 * on the last line.
 *
 * @param[in,out] cex c-style exception
 * @param[in] hd <code>hex_dump</code> instance
 * @param[in] s string to which printable characters are added
 */
static void
cgul_hex_dump__add_printable_characters_to_the_last_line(
    cgul_exception_t* cex,
    cgul_hex_dump_t hd,
    cgul_string_t s)
{
    size_t pad_width = 0;
    cgul_string_t tmp = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(hd);
    assert(s);

    /* If you have some left overs, splice them into s. */
    if( hd->j > 0 ) {

        /* 50 is the width from the start of the hex columns to the
         * end.  I start counting immediately after the colon that
         * follows the byte count thereby including the first space on
         * the line.  The 3 is because each entry takes up 2 places
         * for the hex digits and 1 place for the space that separates
         * entries.  You also have to remember to add in the
         * printable_chars_length as extra because we are adding the
         * padding to that string.  By "padding", I do not mean
         * shifting, I mean printf() style padding.  Also remember,
         * that the number of printable_chars is also the number of
         * hex columns that were used and are now present in the last
         * line of "s" already. */
        pad_width = 50
                    - (hd->j * 3)
                    + cgul_string__get_length(cex, hd->printable_chars);

        /* You also will have to account for the '-' separator that
         * occurs just after the 8th hex value is printed. */
        if( hd->j > 8 ) {
            pad_width -= 2;
        }

        /* Make a copy of the printable_chars strings so that you
         * don't mess it up in case the user decides to call
         * cgul_hex_dump__append() again. */
        tmp = cgul_string__new(cex);

        /* You can either add the the spaces to the end of "s" or to
         * the beginning of "printable_chars".  Here, we add the space
         * to the beginning of "printable_chars".  Actually, we add it
         * to a temporary string so in case the user decides to call
         * cgul_hex_dump__append() again. */
        cgul_string__append(
            cex, tmp, cgul_string__get_value(cex, hd->printable_chars));
        cgul_string__pad_right(
            cex, tmp, pad_width, ' ');

        /* Add the same one space that separate the hex columns from
         * the printable characters column that you have already added
         * to all the other rows. */
        cgul_string__append(cex, s, " ");
        
        /* Finally, append the printable characters. */
        cgul_string__append(cex, s, cgul_string__get_value(cex, tmp));
    }

 out:

    if (tmp) {
        cgul_string__delete(tmp);
    }

    return;
}

cgul_string_t
cgul_hex_dump__get_value(cgul_exception_t* cex,
                         cgul_hex_dump_t hd)
{
    cgul_string_t rv = NULL;
    cgul_list_node_t node = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(hd);

    rv = cgul_string__new(cex);
    cgul_string__set_capacity(cex, rv, hd->total_length);
    if (*cex) {
        goto out;
    }

    node = cgul_list__get_front(cex, hd->strings);
    for( ; node ; node = cgul_list_node__get_next(cex, node) ) {
        cgul_string_t s =
            (cgul_string_t)cgul_list_node__get_value(cex, node);
        cgul_string__append(cex, rv, cgul_string__get_value(cex, s));
        if (*cex) {
            goto out;
        }
    }

    /* The last line needs special treatment. */
    cgul_hex_dump__add_printable_characters_to_the_last_line(cex, hd, rv);
    if (*cex) {
        goto out;
    }

 out:
    return rv;
}

void
cgul_hex_dump__clear(cgul_exception_t* cex,
                     cgul_hex_dump_t hd,
                     int reset_byte_count)
{
    cgul_list_node_t node = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(hd);

    hd->suppress_new_line = 1;
    hd->j = 0;

    if( reset_byte_count ) {
        hd->byte_count = 0;
    }

    hd->total_length = 0;
    cgul_string__set_value(cex, hd->printable_chars, "");

    /* Clear the list including the strings on the list. */
    node = cgul_list__get_front(cex, hd->strings);
    for( ; node ; node = cgul_list_node__get_next(cex, node) ) {
        cgul_string_t s =
            (cgul_string_t)cgul_list_node__get_value(cex, node);
        cgul_string__delete(s);
    }
    cgul_list__clear(cex, hd->strings, NULL);

 out:
    return;
}
