#include "cgul_multiply64.h"

void
cgul_multiply64(cgul_exception_t* cex,
                unsigned int x,
                unsigned int y,
                unsigned int* high,
                unsigned int* low)
{
    unsigned int a = x >> 16;
    unsigned int b = x & 0xffff;
    unsigned int c = y >> 16;
    unsigned int d = y & 0xffff;
    unsigned int hi = 0;  /* high */
    unsigned int lo = 0;  /* low */
    unsigned int tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Do not assert "high" and "low".  They are allowed to be NULL. */

    /*
     * Manually perform the following calculation using one 32-bit
     * types where a, b, c, and d are 16-bit values stored in a 32-bit
     * unsigned type:
     *
     *         x = (a * 2^16 + b)
     *         y = (c * 2^16 + d)
     *
     *     x * y = (a * c * 2^32) + ((a * d + b * c) * 2^16) + (b * d)
     */

    /* Calculate the 2^32 part subject to a potential carries below. */
    hi = a * c;

    /* Calculate the 2^16 part including adding carries to the 2^32 part. */
    tmp = a * d;
    lo = b * c;
    lo += tmp;
    if (lo < tmp) {
        hi += (1 << 16);  /* Carry */
    }

    /* Shift "low" left by 16 bits.  This requires adding the 16
     * high-order bits to "h" first however. */
    hi += (lo >> 16);
    lo = (lo << 16);

    /* Calculate the 2^0 part adding carries to the 2^32 part. */
    tmp = b * d;
    lo += tmp;
    if (lo < tmp) {
        ++hi;  /* Carry */
    }
    
    /* Success. */
    if (high) {
        *high = hi;
    }
    if (low) {
        *low = lo;
    }

 out:
    return;
}
