#ifndef CGUL_RANDOM_LCG256_P_H
#define CGUL_RANDOM_LCG256_P_H

#include "cgul_common.h"
#include "cgul_big_integer.h"
#include "cgul_random_lcg256.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_random_lcg256</code> class.
 */
struct cgul_random_lcg256 {

    /**
     * Current value.
     */
    cgul_big_integer_t curr;

    /**
     * b is one digit smaller than m and ends in x21, where x is even.
     */
    cgul_big_integer_t b;

    /**
     * This is the "m" value common to all LCGs.  It should be one
     * larger than the maximum value you want to be able to generate.
     */
    cgul_big_integer_t m;

    /**
     * This is the mask used as a more efficient way to calculate mod
     * m.  A simple mask with all bits set to 1 can be used because we
     * have chosen m to be a power of 2.
     */
    cgul_big_integer_t mask;

    /**
     * This is the scaled reciprocal of <code>m</code>.  When the code
     * needs to divide by "m", it multiplies by the reciprocal instead.
     */
    cgul_big_integer_t m_reciprocal;

    /**
     * This is the power of 2 to which <code>m_reciprocal</code> is
     * scaled.
     */
    size_t m_scale;

    /**
     * A constant value of 1.
     */
    cgul_big_integer_t one;

    /**
     * Scratch.
     */
    cgul_big_integer_t tmp;

};

CGUL_END_C

#endif  /*  CGUL_RANDOM_LCG256_P_H  */
