#ifndef CGUL_STANZA_P_H
#define CGUL_STANZA_P_H

#include "cgul_common.h"
#include "cgul_stanza.h"

CGUL_BEGIN_C

/** Data encapsulated by the <code>cgul_stanza</code> class. */
struct cgul_stanza {

    /**
     * Treat the file as all one stanza instead of as multiple stanzas
     * separated by vertical white space.
     */
    int all_one_stanza;

    /**
     * Allow duplicate keys.  The last key sets the value.  All
     * previous values will be ignored.
     */
    int allow_duplicate_keys;

    /**
     * Tree of key/value pairs for the current stanza.
     */
    cgul_rbtree_t tree;

    /**
     * The order of the keys on the <code>keys</code> vector matches
     * the order the keys appear in the file for this stanza.  The
     * <code>keys</code> vector merely borrows the keys from the keys
     * in the <code>tree</code> structure.  If
     * <code>allow_duplicate_keys</code> is true, only the first
     * instance of the key is saved to <code>keys</code>.
     */
    cgul_vector_t keys;

    /**
     * List of allowed keys.  Note that we do not own this tree.
     */
    cgul_rbtree_t allowed_keys;

    /**
     * List of mandatory keys.  Note that we do not own this tree.
     */
    cgul_rbtree_t mandatory_keys;

    /**
     * Interface for reading lines from the CSV file regardless of the
     * EOL convention used in the file or data source. <p>
     */
    cgul_crlf_file_t f;

    /**
     * Whether we own <code>f</code>.
     */
    int we_own_f;

    /**
     * Which line in <code>f</code> are we currently on.
     */
    unsigned long int line_number_current;

    /**
     * Which line in <code>f</code> is where the stanza starts.  This
     * is needed because the user just calls
     * <code>cgul_stanza__load_next()</code> which reads in the entire
     * stanza and <code>line_number_current</code> typically points to
     * the bottom of the stanza.  This member variable remembers the
     * line number where the current stanza starts.
     */
    unsigned long int line_number_starting;

    /**
     * Previous key.  If the key for this line is "+", it means that
     * this line is needs to be appended to the previous line in order
     * to generate a multi-line string.  You should never try to free
     * <code>key_prev</code> because it is just a pointer to the
     * previous key which is safely tucked away inside the
     * <code>tree</code>.
     */
    char* prev_key;

};

CGUL_END_C

#endif  /*  CGUL_STANZA_P_H  */
