#include <assert.h>
#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    cgul_bdf_t bdf = NULL;
    cgul_exception_t local = NULL;

    /* Check command line. */
    if (argc != 2) {
        fprintf(stderr, "Usage: main_bdf_cache_expiration <bdf>\n");
        rv = 1;
        goto out;
    }

    /* Allocate bdf. */
    bdf = cgul_bdf__new(&local);
    if (local) {
        goto out;
    }

    /* Set the cache size. */
    cgul_bdf__set_cache_size(&local, bdf, 4);

    /* Load the BDF. */
    cgul_bdf__load(&local, bdf, argv[1]);
    if (local) {
        goto out;
    }

    /* Get some glyphs to test expiration. */
    for (i = 65 ; i < 74 ; ++i) {
        cgul_bdf__get_glyph(&local, bdf, (cgul_wchar_t)i, 0 /* substitution */);
        if (local) {
            goto out;
        }
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (bdf) {
        cgul_bdf__delete(bdf);
    }

    return rv;
}
