#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t max_bits = 0;
    size_t scale = 0;
    cgul_random_bbs_t bbs = NULL;
    cgul_big_integer_t quotient1 = NULL;
    cgul_big_integer_t remainder1 = NULL;
    cgul_big_integer_t quotient2 = NULL;
    cgul_big_integer_t remainder2 = NULL;
    cgul_big_integer_t dividend = NULL;
    cgul_big_integer_t divisor = NULL;
    cgul_big_integer_t reciprocal = NULL;
    cgul_big_integer_t max = NULL;
    cgul_big_integer_t half_max = NULL;
    cgul_big_integer_t tmp = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    bbs = cgul_random_bbs__new(&local);
    quotient1 = cgul_big_integer__new(&local);
    remainder1  = cgul_big_integer__new(&local);
    quotient2 = cgul_big_integer__new(&local);
    remainder2  = cgul_big_integer__new(&local);
    dividend  = cgul_big_integer__new(&local);
    divisor  = cgul_big_integer__new(&local);
    reciprocal  = cgul_big_integer__new(&local);
    max  = cgul_big_integer__new(&local);
    half_max  = cgul_big_integer__new(&local);
    tmp  = cgul_big_integer__new(&local);
    if (local) {
        goto out;
    }

    /* Set the maximum values of the numbers to generate. */
    max_bits = 65536;
    cgul_big_integer__assign_from_long(&local, max, 1);
    cgul_big_integer__shift_left(&local, max, max, max_bits);
    cgul_big_integer__assign_from_long(&local, half_max, 1);
    cgul_big_integer__shift_left(&local, half_max, half_max, max_bits >> 1);
    if (local) {
        goto out;
    }

    for (;;) {

        /* Get the next dividend. */
        cgul_random_bbs__next_in_range(&local, bbs, dividend, max);
        cgul_random_bbs__next_in_range(&local, bbs, tmp, max);
        cgul_big_integer__subtract(&local, dividend, dividend, tmp);
        if (local) {
            goto out;
        }

        do {
            /* Get the next non-zero divisor. */
            cgul_random_bbs__next_in_range(&local, bbs, divisor, half_max);
            cgul_random_bbs__next_in_range(&local, bbs, tmp, half_max);
            cgul_big_integer__subtract(&local, divisor, divisor, tmp);
            if (local) {
                goto out;
            }
        } while(cgul_big_integer__is_zero(&local, divisor));

        /* Print inputs. */
        cgul_big_integer__print_as_hexadecimal(&local, dividend, stdout);
        printf(" / ");
        cgul_big_integer__print_as_hexadecimal(&local, divisor, stdout);
        printf(" = ");
        fflush(stdout);

        /* Divide using long division. */
        cgul_big_integer__divide_long(&local,
                                      quotient1, remainder1,
                                      dividend, divisor);
        if (local) {
            goto out;
        }

        /* Divide using Newton's Method to find the reciprocal. */
        scale = cgul_big_integer__get_inversion_scale(&local,
                                                      dividend,
                                                      divisor);
        cgul_big_integer__invert(&local,
                                 reciprocal,
                                 divisor,
                                 scale);
        cgul_big_integer__divide_newton(&local,
                                        quotient2, remainder2,
                                        dividend, divisor,
                                        reciprocal, scale);
        if (local) {
            goto out;
        }

        /* Compare the results of "long division" and Newton's Method. */
        if (cgul_big_integer__compare(&local, quotient1, quotient2) != 0) {
            local = cgul_exception__new("Quotients disagree!");
            goto out;
        }
        if (cgul_big_integer__compare(&local, remainder1, remainder2) != 0) {
            local = cgul_exception__new("Remainders disagree!");
            goto out;
        }

        /* Print outputs. */
        cgul_big_integer__print_as_hexadecimal(&local, quotient1, stdout);
        printf(" r. ");
        cgul_big_integer__print_as_hexadecimal(&local, remainder1, stdout);
        puts("");
        fflush(stdout);
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "\n*** Error: %s\n\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (tmp) {
        cgul_big_integer__delete(tmp);
    }
    if (half_max) {
        cgul_big_integer__delete(half_max);
    }
    if (max) {
        cgul_big_integer__delete(max);
    }
    if (reciprocal) {
        cgul_big_integer__delete(reciprocal);
    }
    if (divisor) {
        cgul_big_integer__delete(divisor);
    }
    if (dividend) {
        cgul_big_integer__delete(dividend);
    }
    if (remainder2) {
        cgul_big_integer__delete(remainder2);
    }
    if (quotient2) {
        cgul_big_integer__delete(quotient2);
    }
    if (remainder1) {
        cgul_big_integer__delete(remainder1);
    }
    if (quotient1) {
        cgul_big_integer__delete(quotient1);
    }
    if (bbs) {
        cgul_random_bbs__delete(bbs);
    }

    return rv;
}
