#include "cgul.h"

/**
 * Custom string comparison function used as the callback for
 * <code>cgul_binary_search_ceiling()</code> which pass in the address
 * of array element.  For strings, this is the address of a
 * <code>char*</code>.
 *
 * @note By contrast, <code>cgul_binary_search_ceiling_pointers()</code>
 * can directly use <code>strcmp()</code>.
 *
 * @param[in] s1 first string
 * @param[in] s2 second string
 * @return result of comparing strings
 */
static int
main__custom_strcmp(char** s1,
                    char** s2)
{
    return strcmp(*s1, *s2);
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    char* search_key = NULL;
    char* expected = NULL;
    void* actual = NULL;
    char** words = NULL;
    size_t words_size = 0;
    cgul_string_t tmp = NULL;
    cgul_vector_t v = NULL;
    cgul_exception_t local = NULL;
    struct inexact_test_case {
        char* search_key;
        char* expected;
    } inexact_test_cases[] = {
        {"", "aaa"},
        {"a", "aaa"},
        {"b", "bbb"},
        {"c", "ccc"},
        {"l", "lll"},
        {"m", "mmm"},
        {"n", "nnn"},
        {"o", "ooo"},
        {"p", "ppp"},
        {"y", "yyy"},
        {"z", "zzz"},
        {"zzzz", NULL},
    };
    size_t inexact_test_cases_size =
        sizeof(inexact_test_cases) / sizeof(inexact_test_cases[0]);

    /* Allocate. */
    v = cgul_vector__new(&local);
    tmp = cgul_string__new(&local);
    if (local) {
        goto out;
    }

    /* Generate the word list "aaa", "bbb", ... , "zzz". */
    for (i = 'a' ; i <= 'z' ; ++i) {
        cgul_string__append_char(&local, tmp, (char)i);
        cgul_string__append_char(&local, tmp, (char)i);
        cgul_string__append_char(&local, tmp, (char)i);
        cgul_vector__push_back(
            &local, v, cgul_string__get_value(&local, tmp));
        cgul_string__take_value(&local, tmp);
        if (local) {
            goto out;
        }
    }

    /* Unpack the word list. */
    words = (char**)cgul_vector__get_array(&local, v);
    words_size = cgul_vector__get_size(&local, v);

    /*
     * Exact Test Cases (ceiling)
     */

    /* Verify the ceiling function returns all the exact matches. */
    for (i = 0 ; i < words_size ; ++i) {
        search_key = (char*)cgul_vector__at(&local, v, i);
        expected = search_key;
        actual = cgul_binary_search_ceiling(
                    &local,
                    &search_key,
                    (const void**)words,
                    words_size,
                    sizeof(*words),
                    (cgul_binary_search__compare_t)main__custom_strcmp);
        if (!actual || (strcmp(*(char**)actual, search_key) != 0)) {
            local = cgul_exception__new(
                "Exact Test Cases (ceiling): "
                "(search_key, expected, actual) = (-->%s<--, %s, %s)\n",
                search_key,
                expected,
                actual ? *(char**)actual : (char*)actual);
            goto out;
        }
    }

    /*
     * Exact Test Cases (ceiling_pointers)
     */

    /* Verify the ceiling function returns all the exact matches. */
    for (i = 0 ; i < words_size ; ++i) {
        search_key = (char*)cgul_vector__at(&local, v, i);
        expected = search_key;
        actual = cgul_binary_search_ceiling_pointers(
                    &local,
                    search_key,
                    (const void**)words,
                    words_size,
                    (cgul_binary_search__compare_t)strcmp);
        if (!actual || (strcmp((char*)actual, search_key) != 0)) {
            local = cgul_exception__new(
                "Exact Test Cases (ceiling_pointers): "
                "(search_key, expected, actual) = (-->%s<--, %s, %s)\n",
                search_key, expected, (char*)actual);
            goto out;
        }
    }

    /*
     * Inexact Test Cases (ceiling)
     */

    /* Iterate over the inexact test cases. */
    for (i = 0 ; i < inexact_test_cases_size ; ++i) {

        /* Unpack the test case. */
        search_key = inexact_test_cases[i].search_key;
        expected = inexact_test_cases[i].expected;

        /* Execute the test. */
        actual = cgul_binary_search_ceiling(
                    &local,
                    &search_key,
                    (const void**)words,
                    words_size,
                    sizeof(*words),
                    (cgul_binary_search__compare_t)main__custom_strcmp);
        if (expected) {
            /* Verify results when a string is expected. */
            if (!actual || (strcmp(*(char**)actual, expected) != 0)) {
                local = cgul_exception__new(
                    "Inexact Test Cases (ceiling): "
                    "(search_key, expected, actual) = (-->%s<--, %s, %s)\n",
                    search_key,
                    expected,
                    actual ? *(char**)actual : (char*)actual);
                goto out;
            }
        } else {
            /* Verify results when NULL is expected. */
            if (actual) {
                local = cgul_exception__new(
                    "Inexact Test Cases (ceiling): "
                    "(search_key, expected, actual) = (-->%s<--, %p, %p)\n",
                    search_key, expected, actual);
                goto out;
            }
        }

    }

    /*
     * Inexact Test Cases (ceiling_pointers)
     */

    /* Iterate over the inexact test cases. */
    for (i = 0 ; i < inexact_test_cases_size ; ++i) {

        /* Unpack the test case. */
        search_key = inexact_test_cases[i].search_key;
        expected = inexact_test_cases[i].expected;

        /* Execute the test. */
        actual = cgul_binary_search_ceiling_pointers(
                    &local,
                    search_key,
                    (const void**)words,
                    words_size,
                    (cgul_binary_search__compare_t)strcmp);
        if (expected) {
            /* Verify results when a string is expected. */
            if (!actual || (strcmp((char*)actual, expected) != 0)) {
                local = cgul_exception__new(
                    "Inexact Test Cases (ceiling_pointers): "
                    "(search_key, expected, actual) = (-->%s<--, %s, %s)\n",
                    search_key, expected, (char*)actual);
                goto out;
            }
        } else {
            /* Verify results when NULL is expected. */
            if (actual) {
                local = cgul_exception__new(
                    "Inexact Test Cases (ceiling_pointers): "
                    "(search_key, expected, actual) = (-->%s<--, %p, %p)\n",
                    search_key, expected, actual);
                goto out;
            }
        }

    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (tmp) {
        cgul_string__delete(tmp);
    }
    if (v) {
        cgul_vector__free_values(v);
        cgul_vector__delete(v);
    }

    return rv;
}
