#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t rcount = 0;
    size_t block_size = 16384;
    unsigned char* buf = NULL;    
    cgul_block_reader_cxx* br = NULL;

    /* Validate the command line. */
    if (argc != 2) {
        fprintf(stderr, "Usage: main_block_reader_file <file>\n");
        return 1;
    }

    try {

        /* Allocate. */
        buf = (unsigned char*)cgul_libc_cxx::calloc(1, block_size);
        br = new cgul_block_reader_file_cxx(argv[1]);

        for (;;) {

            /* Polymorphically read the next block of data using
             * the cgul_block_reader interface. */
            rcount = br->read_next_block(buf, block_size);
            if (rcount == 0) {
                /* EOF */
                break;
            }

            /* Write the block to stdout. */
            cgul_libc_cxx::fwrite_block(buf, rcount, stdout);
        }
    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    if (br) {
        delete br;
    }
    if (buf) {
        free(buf);
    }

    return rv;
}
