#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t rcount = 0;
    size_t block_size = 1;
    unsigned char* block = NULL;    
    cgul_block_reader_cxx* br = NULL;

    const char* test_string = "Hello, World!\n";

    try {

        /* Allocate. */
        block = (unsigned char*)cgul_libc_cxx::calloc(1, block_size);
        br = new cgul_block_reader_memory_cxx((void*)test_string,
                                              strlen(test_string));

        for (;;) {

            /* Polymorphically read the next block of data using the
             * cgul_block_reader interface. */
            rcount = br->read_next_block(block, block_size);
            if (rcount == 0) {
                /* EOF */
                break;
            }

            /* Write the block to stdout. */
            cgul_libc_cxx::fwrite_block(block, rcount, stdout);
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    if (br) {
        delete br;
    }
    if (block) {
        free(block);
    }

    return rv;
}
