/*
 * This program reads all the lines in text file using one call to the
 * following method:
 *
 *     cgul_crlf_file__get_lines()
 *
 * To run this program do the following:
 *
 *     ./main_crlf_file_get_lines <file>
 */

#include "cgul.h"

/**
 * Print this program's usage message to the file <code>f</code>.  The
 * base name of <code>progname</code> is printed as part of the usage
 * message.
 *
 * @param[in] cex c-style exception
 * @param[in] progname program name
 * @param[in] fout output file
 */
static void
main__print_usage(cgul_exception_t* cex,
                  const char* progname,
                  FILE* fout)
{
    char* bname = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(progname);
    assert(fout);

    /* Get the base name for the program. */
    bname = cgul_libc__basename(cex, progname);
    if (*cex) {
        goto out;
    }

    fprintf(fout, "\n");
    fprintf(fout, "Usage: %s <file>\n", bname);
    fprintf(fout, "\n");

 out:

    /* Clean up. */
    if (bname) {
        free(bname);
    }

    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    char** lines = NULL;
    unsigned long int line_count = 0;
    cgul_crlf_file_t crlf = NULL;
    cgul_exception_t local = NULL;

    /* Validate command-line arguments. */
    if (argc != 2) {
        main__print_usage(&local, argv[0], stderr);
        goto out;
    }

    /* Allocate. */
    crlf = cgul_crlf_file__new_from_fname(
               &local, argv[1], 1 /* is_block_buffered */);
    if (local) {
        goto out;
    }

    /* Read all the lines of text from the file with one call. */
    cgul_crlf_file__get_lines(&local, crlf, &lines, &line_count);
    if (local) {
        goto out;
    }

    /* Print the lines. */
    for (i = 0 ; i < line_count ; ++i) {
        puts(lines[i]);
    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (lines) {
        for (i = 0 ; i < line_count ; ++i) {
            free(lines[i]);
        }
        free(lines);
    }
    if (crlf) {
        cgul_crlf_file__delete(crlf);
    }

    return rv;
}
