#include <cctype>
#include <cstdlib>
#include <cstdio>
#include <stdexcept>
#include <iostream>
#include "cgul_cxx.h"

using namespace std;

static int
get_next_word(FILE* f, cgul_string_cxx& word)
{
    int c = 0;
    int rv = 0;

    /* Clear word. */
    word = "";

    /* Find the first character that is not a white space character. */
    for (;;) {
        c = fgetc(f);
        if (c == EOF) {
            goto out;
        }
        if (!isspace(c)) {
            break;
        }
    }

    word.append_char((char)c);

    /* Find the rest of the characters that comprise the word. */
    for (;;) {
        c = fgetc(f);
        if ((c == EOF) || isspace(c)) {
            break;
        }
        word.append_char((char)c);
    }

    rv = 1;

 out:
    return rv;
}

static void
free_keys(cgul_hash_cxx& h)
{
    cgul_hash_node_cxx* n = h.get_front();
    for ( ; n ; n = n->get_next()) {
        free((void*)n->get_key());
    }
}

static void
print_stats(cgul_hash_cxx& h)
{
    char* tmp = NULL;
    tmp = h.get_stats();
    if (tmp) {
        fflush(NULL);
        fprintf(stderr, "%s\n", tmp);
        free(tmp);
    }
}

static void
print_histogram(cgul_hash_cxx& h)
{
    unsigned long int i = 0;
    unsigned long int value = 0;
    cgul_hash_node_cxx* n = NULL;
    const char** v = NULL;

    /* Get a sorted array of keys. */
    v = (const char**)h.get_sorted_keys(NULL);

    /* Iterate over the keys. */
    for (i = 0 ; v[i] ; ++i) {
        if ((n = h.find(v[i])) != NULL) {
            value = (unsigned long int)n->get_value();
            printf("%s  %lu\n", v[i], value);
        }
    }

    free(v);

    return;
}

static void
add_to_histogram(cgul_hash_cxx& h, cgul_string_cxx& s) {
    unsigned long int count = 0;
    char* key = NULL;
    cgul_hash_node_cxx* n = NULL;

    key = s.get_value();
    if (h.insert(key, &n)) {
        /* Let cgul_string know we've taken ownership of key. */
        s.take_value();
        n->set_value((const void*)1);
    } else {
        count = (unsigned long int)n->get_value();
        n->set_value((const void*)++count);
    }
}

int main(int argc, char* argv[])
{
    int rv = 0;
    cgul_hash_cxx h;
    cgul_string_cxx s;

    try {

        while (get_next_word(stdin, s)) {
            add_to_histogram(h, s);
        }

        print_histogram(h);
        print_stats(h);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    free_keys(h);

    return rv;
}
