#include <stdio.h>
#include "cgul.h"

static int
free_hash_keys(cgul_exception_t* cex,
               cgul_hash_t h,
               cgul_hash_node_t n,
               void* data)
{
    int rv = 0;
    char* key = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    key = (char*)cgul_hash_node__get_key(cex, n);
    free(key);

    rv = 1;

 out:
    return rv;
}

int main(int argc, char* argv[])
{
    int rv = 0;
    int i = 0;
    char* key = NULL;
    cgul_hash_t h = NULL;
    cgul_string_t s = NULL;
    cgul_hash_node_t n = NULL;
    cgul_exception_t local = NULL;

    s = cgul_string__new(&local);
    h = cgul_hash__new(&local);

    if (local) {
        goto out;
    }

    /* Insert keys for "0" through "99". */
    for (i = 0 ; i < 100 ; ++i) {
        cgul_string__set_value(&local, s, "");
        cgul_string__append_decimal(&local, s, i);
        key = cgul_string__get_value(&local, s);
        if (cgul_hash__insert(&local, h, key, NULL)) {
            cgul_string__take_value(&local, s);
        } else {
            fprintf(stderr, "*** Error: Unable to insert key: %s\n", key);
        }
    }

    if (local) {
        goto out;
    }

    /* Print keys forward. */
    n = cgul_hash__get_oldest(&local, h);
    while (n) {
        key = (char*)cgul_hash_node__get_key(&local, n);
        puts(key);
        n = cgul_hash_node__get_younger(&local, n);
    }

    cgul_libc__puts(&local, "=====");

    /* Print keys reverse. */
    n = cgul_hash__get_youngest(&local, h);
    while (n) {
        key = (char*)cgul_hash_node__get_key(&local, n);
        puts(key);
        n = cgul_hash_node__get_older(&local, n);
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** main_hash: Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (h) {
        cgul_hash__traverse(&local, h, free_hash_keys, NULL);
        cgul_hash__delete(h);
    }
    if (s) {
        cgul_string__delete(s);
    }

    return rv;
}
