/*
 * This test program loads a file into memory using
 * cgul_libc__load_fname() and immediately writes it back out to disk
 * using cgul_libc__save_fname().  The user specifies both file names
 * on the command line when invoking this program.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t block_size = 0;
    char* block = NULL;
    cgul_exception_t local = NULL;

    /* Verify command line. */
    if (argc != 3) {
        fprintf(stderr, "Usage: main_libc_save_file <fname_in> <fname_out>\n");
        goto out;
    }

    /* Load the file into memory. */
    cgul_libc__load_fname(&local, argv[1], &block, &block_size);
    if (local) {
        goto out;
    }

    /* Save the memory to file. */
    cgul_libc__save_fname(&local, argv[2], block, block_size);
    if (local) {
        goto out;
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (block) {
        free(block);
    }

    return rv;
}
