/**
 * This program generates 250K lines of random c-style ascii strings.
 * These strings are limited to the ascii character set which for this
 * program means the following inclusive range of charactes:
 *
 *     [32 - 126] + '\n'
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    unsigned long int c = 0;
    cgul_random_lcg256_t prng = NULL;
    cgul_big_integer_t max = NULL;
    cgul_big_integer_t next = NULL;
    cgul_exception_t local = NULL;

    /* Allocate.  Note that we need to set "max" to include a byte for
     * the '\n' character which out side of the normal range. */
    prng = cgul_random_lcg256__new(&local);
    max = cgul_big_integer__new_from_string(&local, "96");
    next = cgul_big_integer__new(&local);
    if (local) {
        goto out;
    }

    /* Generate 250K lines of c-style strings.  The only thing
     * different about this from having all random characters is that
     * each zero that is generated is replaced with '\n' so that the
     * file can be directly read into a C program. */
    while (i < 256000) {
        /* Get the next random number in the range [0 - 95]. */
        cgul_random_lcg256__next_in_range(&local, prng, next, max);
        if (!cgul_big_integer__as_unsigned_long(&local, next, &c)) {
            local = cgul_exception__new("out of range");
            goto out;
        }
        /* Map to the range [32 - 127]. */
        c += 32;
        /* The character at 127 counts as '\n' because it outside of
         * the normal range so we map 127 to '\n'. */
        if (c == 127) {
            c = '\n';
            ++i;
        }
        /* Print this character. */
        putc((int)c, stdout);
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (max) {
        cgul_big_integer__delete(max);
    }
    if (next) {
        cgul_big_integer__delete(next);
    }
    if (prng) {
        cgul_random_lcg256__delete(prng);
    }

    return rv;
}
