#include <assert.h>
#include "cgul.h"

static void
print_tree(cgul_exception_t* cex, cgul_rbtree_t tree)
{
    cgul_rbtree_node_t curr = NULL;
    cgul_string_t s = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(tree);

    cgul_libc__printf(cex, "---\n");

    for (curr = cgul_rbtree__get_front(cex, tree) ;
         (*cex == NULL) && curr ;
         curr = cgul_rbtree_node__get_next(cex, curr))
    {
        s = (cgul_string_t)cgul_rbtree_node__get_key(cex, curr);
        cgul_libc__printf(cex, "%s\n", cgul_string__get_value(cex, s));
    }

 out:
    return;
}

int main(int argc, char* argv[])
{
    int rv = 0;
    int i = 0;
    cgul_string_t s = NULL;
    cgul_cache_t strings_cache = NULL;
    cgul_rbtree_t tree = NULL;
    cgul_rbtree_node_t node = NULL;
    cgul_exception_t local = NULL;

    /* strings_cache */
    strings_cache = 
        cgul_cache__new(
            &local,
            (cgul_cache__constructor_t) &cgul_string__new,
            (cgul_cache__destructor_t) &cgul_string__delete,
            (cgul_cache__restructor_t) &cgul_string__clear);
    cgul_cache__set_size(&local, strings_cache, 16);

    /* tree */
    tree = cgul_rbtree__new(&local,
                             (cgul_rbtree__compare_t)cgul_string__compare);
    cgul_rbtree__set_cache_size(&local, tree, 16);

    /* Add keys to tree. */
    for (i = 0 ; (local == NULL) && (i < 8) ; ++i) {
        s = cgul_cache__get(&local, strings_cache);
        cgul_string__append_sprintf(&local, s, "%d", i);
        cgul_rbtree__insert(&local, tree, s, NULL);
    }

    /* Print the tree. */
    print_tree(&local, tree);

    /* Remove back node using remove_range(). */
    node = cgul_rbtree__get_back(&local, tree);
    cgul_rbtree__remove_range(&local, tree, node, node, strings_cache, NULL);

    /* Print the tree. */
    print_tree(&local, tree);

    /* Remove nodes from "4" to end of tree. */
    s = cgul_cache__get(&local, strings_cache);
    cgul_string__set_value(&local, s, "4");
    node = cgul_rbtree__find(&local, tree, s);
    if (node) {
        cgul_rbtree__remove_range(&local, tree,
                                   node, NULL,
                                   strings_cache, NULL);
    }
    cgul_cache__put(&local, strings_cache, s);

    /* Print the tree. */
    print_tree(&local, tree);

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    if (tree) {
        cgul_rbtree__clear(&local, tree, strings_cache, NULL);
        cgul_rbtree__delete(tree);
    }

    if (strings_cache) {
        cgul_cache__delete(strings_cache);
    }

    return rv;
}
