#include "cgul.h"

/**
 * Print the keys in the tree <code>t</code>.
 *
 * @param[in] cex c-style exception
 * @param[in] t <code>cgul_rbtree</code> instance
 */
static void
main__print_keys(cgul_exception_t* cex,
                 cgul_rbtree_t t)
{
    const char* key = NULL;
    cgul_rbtree_node_t curr = NULL;
    
    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Iterate over each key. */
    curr = cgul_rbtree__get_front(cex, t);
    for ( ; curr ; curr = cgul_rbtree_node__get_next(cex, curr)) {
        key = cgul_rbtree_node__get_key(cex, curr);
        printf("%s\n", key);
    }

 out:
    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_rbtree_t t1 = NULL;
    cgul_rbtree_t t2 = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    t1 = cgul_rbtree__new(&local, (cgul_rbtree__compare_t)strcmp);
    t2 = cgul_rbtree__new(&local, (cgul_rbtree__compare_t)strcmp);
    if (local) {
        goto out;
    }

    /* Insert keys into t1. */
    cgul_rbtree__insert(&local, t1, "aaa", NULL);
    cgul_rbtree__insert(&local, t1, "bbb", NULL);
    cgul_rbtree__insert(&local, t1, "ccc", NULL);

    /* Insert keys into t2. */
    cgul_rbtree__insert(&local, t2, "xxx", NULL);
    cgul_rbtree__insert(&local, t2, "yyy", NULL);
    cgul_rbtree__insert(&local, t2, "zzz", NULL);

    /* Print keys originally in t1. */
    printf("===== t1 (before)\n");
    main__print_keys(&local, t1);

    /* Print keys originally in t2. */
    printf("===== t2 (before)\n");
    main__print_keys(&local, t2);

    /* Swap */
    cgul_rbtree__swap(&local, t1, t2);

    /* Print keys originally in t1. */
    printf("===== t1 (after)\n");
    main__print_keys(&local, t1);

    /* Print keys originally in t2. */
    printf("===== t2 (after)\n");
    main__print_keys(&local, t2);

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = NULL;
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (t2) {
        cgul_rbtree__delete(t2);
    }
    if (t1) {
        cgul_rbtree__delete(t1);
    }

    return rv;
}
