#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    size_t i_max = 0;
    size_t length = 0;
    cgul_uint32_t hv_rolling = 0;
    cgul_uint32_t hv_substring = 0;
    const char* s = "Hello, Cruel World!  Goodbye, Cruel World!";
    const char* substring = "Goodbye, Cruel World!";
    cgul_exception_t local = NULL;

    /* Determine the length of the substring.  This will be used as
     * the length of the window for the rolling hash. */
    length = strlen(substring);

    /* Directly calculate the hash value for the substring. */
    hv_substring = cgul_rolling_hash__hash(&local, substring, length);
    if (local) {
        goto out;
    }

    /* Print the hash value for the substring. */
    printf("hv_substring = %10lu\n", (unsigned long)hv_substring);

    /* Seed the rolling hash by directly calculating the hash value
     * for the first "length" bytes in s. */
    hv_rolling = cgul_rolling_hash__hash(&local, s, length);
    if (local) {
        goto out;
    }

    /* Print the starting value of the rolling hash. */
    printf("  hv_rolling = %10lu\n", (unsigned long)hv_rolling);

    /* Calculate the rolling hash until we get to the end of the string. */
    i_max = strlen(s);
    for (i = 0 ; i + length < i_max ; ++i) {
        
        /* Calculate the next hash for the rolling window. */
        hv_rolling = cgul_rolling_hash__roll(&local,
                                             hv_rolling,
                                             s[i],
                                             s[i + length],
                                             length);
        if (local) {
            goto out;
        }

        /* Print the current value of the rolling hash. */
        printf("  hv_rolling = %10lu\n", (unsigned long)hv_rolling);
    }

    /* Check if the final rolling hash value matches the directly
     * calculated hash value for the substring at the end of the
     * string "s". */
    if (hv_rolling == hv_substring) {
        printf("Success!\n");
    } else {
        printf("Failure!\n");
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    return rv;
}
