#include <cassert>
#include <cstring>
#include <iostream>
#include <stdexcept>
#include <string>
#include "cgul_cxx.h"

using namespace std;

/**
 * In-memory stanzas.
 */
static string memory = "\
#\n\
# This is the input file used to test main_stanza.c.\n\
#\n\
\n\
# Stanza #1.\n\
foo: value 1\n\
bar: value 2\n\
baz: value 3 (1st line)\n\
  +: value 3 (2nd line)\n\
  +: value 3 (3rd line)\n\
\n\
    # Stanza #2.  Notice that indentation is allowed.\n\
    foo: value 4\n\
    bar: value 5\n\
    baz: value 6\n\
    # The \"hmm\" key is allowed but not mandatory, i.e., it is optional.\n\
    hmm: value 7\n\
\n\
\n\
# Notice that the spaces on the next line do not prevent it from being\n\
# treated as a blank line.\n\
     \n\
\n\
\n\
# Notice also that it is ok to have more than one blank line between\n\
# stanzas.\n\
#\n\
# Stanza #3.\n\
foo: value 8\n\
bar: value 9\n\
baz: value 10 (IN-MEMORY!!!)\n\
";

/**
 * This method prints the key/value pair for <code>key</code>.
 *
 * @param[in] stanza stanza
 * @param[in] key key
 */
static void
print_key_value_pair(const cgul_stanza_cxx& stanza,
                     const char* key)
{
    const char* value = NULL;

    value = stanza.lookup(key);
    if (value == NULL) {
        value = "<invalid key>";
    }
    cout << key << " ==> " << value << endl;
}

int main(int argc, char* argv[])
{
    int rv = 0;

    try {

        /* Open the stanza text "file" in memory. */
        cgul_stanza_cxx stanza(memory.c_str(), memory.length());

    /* Iterate over each stanza. */
        while (stanza.load_next()) {
            print_key_value_pair(stanza, "foo");
            print_key_value_pair(stanza, "bar");
            print_key_value_pair(stanza, "baz");
            print_key_value_pair(stanza, "hmm");
            cout << "====" << endl;
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
