#include <cerrno>
#include <cstring>
#include <iostream>
#include <sstream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

/**
 * This method prints the key/value pair for <code>key</code>.
 *
 * @param[in] stanza stanza
 * @param[in] key key
 */
static void
print_key_value_pair(cgul_stanza_cxx& stanza,
                     const char* key)
{
    const char* value = NULL;

    assert(key);

    /* Lookup the value. */
    value = stanza.lookup(key);

    /* Carefully print the key/value pair. */
    cout << key << " ==> " << (value ? value : "<invalid key>") << endl;
}

/**
 * Print each stanza from the stanza object <code>stanza</code>.  If
 * an error occurs, an exception is thrown.
 *
 * @param[in] stanza <code>cgul_stanza</code> instance
 */
static void
print_stanzas(cgul_stanza_cxx& stanza)
{
    /* Iterate over each stanza. */
    while (stanza.load_next()) {
        print_key_value_pair(stanza, "foo");
        print_key_value_pair(stanza, "bar");
        print_key_value_pair(stanza, "baz");
        print_key_value_pair(stanza, "hmm");
        cout << "====" << endl;
    }
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    FILE* f = NULL;

    try {

        cgul_crlf_file_cxx crlf_file;
        cgul_stanza_cxx stanza;

        /* Open the stanza file and print its stanzas. */
        stanza.open_fname("main_stanza_input.txt", 1 /* is_block_buffered */);
        print_stanzas(stanza);
        puts("**************************************************************");

        /* Re-open the stanza file and print its stanzas. */
        stanza.open_fname("main_stanza_input.txt", 1 /* is_block_buffered */);
        print_stanzas(stanza);
        puts("**************************************************************");

        /* Re-open the stanza file and print its stanzas. */
        f = fopen("main_stanza_input.txt", "r");
        if (!f) {
            ostringstream msg;
            msg << "fopen: " << strerror(errno);
            throw runtime_error(msg.str());
        }
        stanza.open_file(f, 1 /* is_block_buffered */);
        print_stanzas(stanza);
        puts("**************************************************************");

        /* Re-open the stanza file and print its stanzas. */
        crlf_file.open_fname("main_stanza_input.txt", 1 /* is_block_buffered */);
        stanza.open_crlf_file(crlf_file);
        print_stanzas(stanza);
        puts("**************************************************************");

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    }

    /* Clean up. */
    if (f) {
        fclose(f);
    }

    return rv;
}
