#include "cgul.h"

static int
print_value(cgul_exception_t* cex,
            void* value,
            void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("%d\n", *(int*)value);
    rv = 1;

 out:
    return rv;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    int* value = NULL;
    cgul_vector_t v = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    v = cgul_vector__new(&local);
    if (local) {
        goto out;
    }

    /* Populate the vector. */
    for (i = 0 ; i < 5 ; ++i) {
        /* Allocate a new value to add to the vector. */
        value = cgul_libc__calloc(&local, 1, sizeof(*value));
        if (local) {
            goto out;
        }
        /* Assign a value to the newly allocated memory. */
        *value = i;
        /* Add the value to the vector. */
        cgul_vector__push_back(&local, v, value);
        if (local) {
            goto out;
        }
    }

    /* Print the values on the vector in forward and then reverse order. */
    cgul_vector__foldl(&local, v, print_value, NULL);
    puts("=====");
    cgul_vector__foldr(&local, v, print_value, NULL);

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (v) {
        cgul_vector__free_values(v);
        cgul_vector__delete(v);
    }

    return rv;
}
