#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

struct point {
    int x;
    int y;
};

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i;
    size_t v_count = 10;
    struct point* pt = NULL;
    cgul_vector_cxx v;
    cgul_random_lfg32_cxx prng;

    try {

        /* Initialize. */
        for (i = 0 ; i < v_count ; ++i) {
            /* Allocate and initialize a new point. */
            pt = (struct point*)cgul_libc_cxx::calloc(1, sizeof(*pt));
            pt->x = i;
            pt->y = i + 1;

            /* Add the point to the vector. */
            v.push_back(pt);

            /* The vector now owns the point. */
            pt = NULL;
        }

        /* Shuffle. */
        v.shuffle((cgul_vector_cxx::next_in_range_t)
                      cgul_random_lfg32__next_in_range,
                  prng.get_obj());

#if 1
        {
            size_t i_max = 0;
            struct point** arr = NULL;

            /* Print the shuffled array. */
            arr = (struct point**)v.get_array();
            i_max = v.get_size();
            for (i = 0 ; i < i_max ; ++i) {
                printf("(%6d, %6d)\n", arr[i]->x, arr[i]->y);
            }
        }
#endif

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    if (pt) {
        free(pt);
    }
    v.free_values();

    return rv;
}
